<?php
 namespace App\Controller\Frontend; use Symfony\Component\HttpFoundation\Request; use Symfony\Component\HttpFoundation\Response; use Symfony\Component\Form\Form; use Symfony\Component\Form\Extension\Core\Type\SubmitType; use Symfony\Contracts\Translation\TranslatorInterface; use Symfony\Component\Validator\Validator\ValidatorInterface; use Symfony\Component\Validator\ConstraintViolationList; use App\Controller\Controller; use App\Event\EventQueue; use App\Service\Logger; use App\Entity\Site as SiteEntity; use App\Entity\Certificate as CertificateEntity; use App\Entity\NodejsSettings as NodejsSettingsEntity; use App\Entity\PhpSettings as PhpSettingsEntity; use App\Entity\PythonSettings as PythonSettingsEntity; use App\Entity\Database as DatabaseEntity; use App\Entity\DatabaseUser as DatabaseUserEntity; use App\Entity\DatabaseServer as DatabaseServerEntity; use App\Entity\User as UserEntity; use App\Entity\Manager\ConfigManager; use App\Entity\Manager\SiteManager as SiteEntityManager; use App\Entity\Manager\CertificateManager as CertificateEntityManager; use App\Entity\Manager\DatabaseManager as DatabaseEntityManager; use App\Entity\Manager\DatabaseUserManager as DatabaseUserEntityManager; use App\Entity\Manager\DatabaseServerManager as DatabaseServerEntityManager; use App\Entity\Manager\VhostTemplateManager as VhostTemplateEntityManager; use App\Database\Connection as DatabaseConnection; use App\Database\Manager as DatabaseManager; use App\Site\Nginx\Vhost\StaticTemplate as StaticVhostTemplate; use App\Site\Nginx\Vhost\PhpTemplate as PhpVhostTemplate; use App\Site\Nginx\Vhost\PythonTemplate as PythonVhostTemplate; use App\Site\Nginx\Vhost\ReverseProxyTemplate as ReverseProxyVhostTemplate; use App\Site\Nginx\Vhost\Processor\RedirectDomain as RedirectDomainProcessor; use App\Site\Nginx\Vhost\Processor\RedirectServerName as RedirectServerNameProcessor; use App\Site\Nginx\Vhost\Processor\ServerName as ServerNameProcessor; use App\Site\Parser\DomainName as DomainNameParser; use App\Site\Ssl\Generator\RsaKeyGenerator; use App\Site\Creator\NodejsSite as NodejsSiteCreator; use App\Site\Creator\StaticSite as StaticSiteCreator; use App\Site\Creator\PhpSite as PhpSiteCreator; use App\Site\Creator\PythonSite as PythonSiteCreator; use App\Site\Creator\ReverseProxySite as ReverseProxySiteCreator; use App\Site\Ssl\DistinguishedName; use App\Site\Ssl\Generator\CsrGenerator; use App\Site\Ssl\Util\Openssl; use App\Site\NodejsSite; use App\Site\StaticSite; use App\Site\PhpSite; use App\Site\PythonSite; use App\Site\ReverseProxySite; use App\Util\PasswordGenerator; use App\Site\Application\WordPressInstaller; use App\Service\Crypto; class NewSiteController extends Controller { private const WORDPRESS_VHOST_TEMPLATE_NAME = "\x57\157\x72\x64\120\x72\145\163\x73"; private const NODEJS_VHOST_TEMPLATE_NAME = "\116\x6f\144\x65\x6a\163"; private const STATIC_SITE_VHOST_TEMPLATE_NAME = "\123\164\141\x74\151\143"; private const PYTHON_VHOST_TEMPLATE_NAME = "\120\171\x74\150\x6f\156"; private const REVERSE_PROXY_VHOST_TEMPLATE_NAME = "\122\145\166\x65\x72\163\x65\x50\x72\x6f\170\x79"; private const VARNISH_CACHE_SERVER = "\61\x32\x37\x2e\60\56\x30\x2e\x31\72\66\x30\x38\61"; private const PASSWORD_LENGTH = 20; private ConfigManager $configManager; private SiteEntityManager $siteEntityManager; private CertificateEntityManager $certificateEntityManager; private DatabaseEntityManager $databaseEntityManager; private DatabaseUserEntityManager $databaseUserEntityManager; private DatabaseServerEntityManager $databaseServerEntityManager; private DomainNameParser $domainNameParser; private VhostTemplateEntityManager $vhostTemplateEntityManager; private ValidatorInterface $validator; private array $wordPressDefaultConfigValues = ["\106\x53\137\x4d\105\x54\x48\x4f\x44" => ["\166\141\x6c\165\145" => "\144\151\x72\145\x63\x74", "\162\x61\x77" => false], "\x57\x50\x5f\104\x45\102\125\107\x5f\x44\111\123\x50\x4c\101\x59" => ["\166\x61\154\165\145" => "\x66\x61\x6c\163\145", "\162\141\x77" => true], "\x57\x50\x5f\104\105\102\125\x47\137\x4c\x4f\x47" => ["\166\x61\x6c\x75\x65" => "\x74\x72\x75\x65", "\x72\141\167" => true], "\103\117\x4e\x43\101\x54\x45\116\x41\124\x45\x5f\123\103\x52\111\120\x54\x53" => ["\x76\x61\x6c\x75\x65" => "\146\141\154\x73\145", "\x72\x61\x77" => true], "\x41\125\x54\117\123\101\x56\x45\137\x49\x4e\124\x45\122\126\101\114" => ["\x76\x61\x6c\165\145" => 600, "\162\141\167" => true], "\127\x50\137\x50\x4f\123\124\137\122\105\x56\x49\123\x49\117\116\x53" => ["\x76\141\x6c\165\145" => 5, "\x72\x61\167" => true], "\x45\115\120\124\x59\x5f\x54\x52\x41\x53\x48\x5f\x44\101\131\x53" => ["\x76\141\x6c\x75\x65" => 21, "\162\141\167" => true]]; private array $wordPressMultiSiteConfigValues = ["\x57\120\137\x41\114\x4c\117\x57\137\x4d\x55\114\x54\111\123\111\124\105" => ["\166\x61\154\x75\145" => "\164\x72\x75\145", "\162\141\x77" => true], "\115\x55\114\124\x49\123\x49\x54\105" => ["\x76\x61\154\x75\x65" => "\164\162\x75\145", "\x72\x61\x77" => true], "\x53\125\102\104\117\115\101\111\116\x5f\111\116\x53\124\x41\114\114" => ["\166\x61\154\165\145" => "\146\141\154\163\145", "\x72\x61\x77" => true], "\x44\x4f\115\101\x49\116\137\103\125\122\x52\x45\116\124\137\123\x49\x54\105" => ["\166\141\x6c\x75\x65" => '', "\x72\141\x77" => false], "\120\x41\124\x48\x5f\x43\125\x52\x52\105\x4e\x54\137\123\x49\124\105" => ["\x76\141\x6c\165\x65" => "\57", "\162\141\167" => false], "\123\x49\x54\x45\137\111\x44\x5f\103\125\122\x52\x45\x4e\124\x5f\x53\x49\124\105" => ["\x76\x61\x6c\x75\145" => 1, "\x72\141\167" => true], "\102\x4c\117\x47\x5f\x49\x44\x5f\103\125\122\x52\105\x4e\124\137\x53\111\x54\105" => ["\x76\x61\x6c\165\145" => 1, "\162\x61\x77" => true]]; public function __construct(ConfigManager $configManager, SiteEntityManager $siteEntityManager, CertificateEntityManager $certificateEntityManager, DatabaseEntityManager $databaseEntityManager, DatabaseUserEntityManager $databaseUserEntityManager, DatabaseServerEntityManager $databaseServerEntityManager, VhostTemplateEntityManager $vhostTemplateEntityManager, DomainNameParser $domainNameParser, ValidatorInterface $validator, TranslatorInterface $translator, Logger $logger) { goto a3e77; Ed1d3: $this->databaseServerEntityManager = $databaseServerEntityManager; goto D1196; a3e77: $this->configManager = $configManager; goto E24f1; c283d: $this->certificateEntityManager = $certificateEntityManager; goto F9fac; ff527: $this->databaseUserEntityManager = $databaseUserEntityManager; goto Ed1d3; E24f1: $this->siteEntityManager = $siteEntityManager; goto c283d; F9fac: $this->databaseEntityManager = $databaseEntityManager; goto ff527; ac0cc: parent::__construct($translator, $logger); goto E3582; b48a9: $this->validator = $validator; goto ac0cc; D1196: $this->vhostTemplateEntityManager = $vhostTemplateEntityManager; goto af5b1; af5b1: $this->domainNameParser = $domainNameParser; goto b48a9; E3582: } public function index(Request $request) : Response { goto C4134; b6df9: $response = $this->redirect($this->generateUrl("\143\154\x70\137\163\151\164\145\163")); goto Ce47f; a5aa6: $user = $this->getUser(); goto Cfbf7; Cfbf7: $response = $this->render("\106\x72\x6f\156\x74\145\156\x64\x2f\x53\151\164\145\x2f\x4e\x65\167\x2f\151\156\x64\x65\x78\x2e\150\x74\x6d\154\x2e\x74\x77\x69\x67", ["\x75\163\145\x72" => $user]); goto C72e1; Bfefc: return $response; goto C9e47; fc168: if (true === $hasPermissions) { goto c5711; } goto b6df9; C4134: $hasPermissions = $this->hasPermissions(); goto fc168; C72e1: c9d9a: goto Bfefc; Ffbba: c5711: goto a5aa6; Ce47f: goto c9d9a; goto Ffbba; C9e47: } public function newPythonSite(Request $request) : Response { goto E8285; B9fc0: $response = $this->handleCreatePythonSiteForm($request, $form); goto a4d5b; Ac291: De5d8: goto c0844; c0844: E1d1c: goto Fb00f; A1ad1: $user = $this->getUser(); goto Fd187; E8285: $hasPermissions = $this->hasPermissions(); goto b7717; dc351: return $response; goto cefd3; E2a3e: if (!(true === $form->isSubmitted())) { goto E1d1c; } goto B9fc0; f4f0e: $form->handleRequest($request); goto E2a3e; b7717: if (true === $hasPermissions) { goto f9d88; } goto D7816; Da5dd: $form = $this->createPythonSiteForm($pythonSettings); goto d9659; d9659: if (!(true === $request->isMethod("\120\x4f\123\x54"))) { goto e4e1c; } goto f4f0e; Fb00f: e4e1c: goto d77b1; bebe5: f9d88: goto A1ad1; cc75b: goto Aeffd; goto bebe5; eacc2: Aeffd: goto dc351; D7816: $response = $this->redirect($this->generateUrl("\x63\154\160\x5f\163\151\164\145\163")); goto cc75b; a4d5b: if (!(false === is_null($response))) { goto De5d8; } goto d1cfc; d77b1: $response = $this->render("\x46\x72\x6f\156\x74\145\x6e\x64\x2f\x53\151\x74\145\x2f\x4e\x65\167\x2f\x70\x79\x74\x68\x6f\156\x2e\150\x74\x6d\154\x2e\164\167\151\x67", ["\165\x73\x65\162" => $user, "\146\157\162\155" => $form->createView(), "\146\x6f\162\x6d\x45\x72\x72\x6f\x72\163" => $this->formErrors]); goto eacc2; d1cfc: return $response; goto Ac291; Fd187: $pythonSettings = new PythonSettingsEntity(); goto Da5dd; cefd3: } private function createPythonSiteForm(PythonSettingsEntity $pythonSettings) : Form { goto E2808; e09eb: $form->add("\163\x75\x62\155\151\164", SubmitType::class, ["\141\x74\164\x72" => ["\143\154\x61\163\x73" => "\142\x74\156\x20\142\x74\156\55\154\147\x20\x62\x74\156\55\x62\154\165\x65"], "\x6c\x61\x62\x65\x6c" => "\x43\x72\145\141\164\x65"]); goto Dbb74; Dbb74: return $form; goto e92fe; E2808: $form = $this->createForm("\x41\x70\160\134\106\x6f\x72\x6d\134\x53\x69\x74\x65\x4e\145\x77\120\x79\164\150\x6f\x6e\124\171\x70\x65", $pythonSettings, ["\141\x63\x74\x69\157\x6e" => $this->generateUrl("\x63\154\160\x5f\163\151\164\x65\137\x70\x79\x74\x68\157\156\x5f\156\x65\167"), "\155\145\x74\150\x6f\x64" => "\120\117\x53\124", "\x61\x74\x74\162" => ["\151\x64" => "\x6e\x65\167\55\x70\x79\164\150\157\156\55\x73\x69\x74\x65\x2d\146\x6f\x72\155"]]); goto e09eb; e92fe: } private function handleCreatePythonSiteForm(Request $request, Form $form) { goto C933b; C933b: if (true === $form->isValid()) { goto a81d9; } goto f8213; f8213: $this->formErrors = $this->getErrorMessages($form); goto c8ba8; B00a7: a81d9: goto B8f76; B8f76: try { goto b684d; a5be8: $pythonVersion = $form->get("\160\x79\x74\150\157\x6e\x56\145\162\x73\x69\157\x6e")->getData(); goto D3b06; De7d3: if (false === is_null($vhostTemplateEntity)) { goto E4540; } goto c4379; bae8c: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto De7d3; db733: EventQueue::addEvent(EventQueue::EVENT_SITE_PYTHON_CREATE, $user, $eventData, $request); goto A5d98; B847f: $domainName = $form->get("\x64\157\x6d\x61\x69\156\116\x61\x6d\145")->getData(); goto a5be8; c4379: throw new \Exception(sprintf("\x56\x68\x6f\x73\164\x20\x74\145\x6d\160\x6c\141\x74\x65\x20\42\45\163\x22\x20\x6e\157\164\40\x66\x6f\165\x6e\x64\x2e", $vhostTemplateName)); goto Ebc9e; C8f61: $siteUserPassword = $form->get("\x73\151\164\145\x55\x73\145\x72\x50\141\163\x73\x77\157\162\144")->getData(); goto fdc1d; A5d98: $session->getFlashBag()->set("\163\165\x63\143\145\x73\x73", $this->translator->trans("\123\151\164\145\40\150\x61\x73\40\142\145\x65\x6e\40\x63\162\145\141\x74\x65\x64\56")); goto e041a; D167f: $siteUser = $form->get("\163\x69\164\x65\125\163\145\162")->getData(); goto C8f61; e6a53: e6868: goto F5ef8; a6990: return $response; goto e6a53; Ebc9e: goto e6868; goto b0219; b0219: E4540: goto B10cf; C48c8: $this->createPythonSite($domainName, $pythonVersion, $port, $siteUser, $siteUserPassword); goto f2409; b684d: $session = $request->getSession(); goto B847f; D3b06: $port = $form->get("\x70\157\x72\x74")->getData(); goto D167f; B10cf: $user = $this->getUser(); goto C48c8; e041a: $response = $this->redirect($this->generateUrl("\x63\x6c\160\137\x73\x69\164\x65\163")); goto a6990; fdc1d: $vhostTemplateName = self::PYTHON_VHOST_TEMPLATE_NAME; goto bae8c; f2409: $eventData = ["\x73\151\164\145" => $domainName, "\x73\151\x74\145\x55\163\145\162" => $siteUser, "\160\171\x74\150\x6f\x6e\x56\x65\162\163\x69\x6f\x6e" => $pythonVersion, "\160\157\162\164" => $port]; goto db733; F5ef8: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\144\141\x6e\x67\x65\x72", $this->translator->trans("\x41\x6e\x20\145\162\162\x6f\162\x20\x68\141\163\40\157\x63\x63\165\162\162\145\144\x2c\x20\x65\162\x72\157\x72\40\x6d\145\163\163\x61\x67\145\x3a\x20\x25\145\162\162\x6f\x72\115\x65\163\163\141\x67\145\x25", ["\x25\x65\x72\162\x6f\x72\x4d\x65\x73\x73\141\x67\x65\45" => $e->getMessage()])); } goto a1e69; a1e69: F4870: goto b546b; c8ba8: goto F4870; goto B00a7; b546b: } public function newReverseProxy(Request $request) : Response { goto b68ee; d0442: E9114: goto E751b; F90e4: return $response; goto e92db; Cce9c: goto efa31; goto E5a28; D611a: $form = $this->createReverseProxyForm(); goto B01cd; acf57: $form->handleRequest($request); goto Ecc92; F36e0: $response = $this->redirect($this->generateUrl("\143\154\x70\137\163\151\164\145\x73")); goto Cce9c; cf42d: return $response; goto c9fbd; df898: efa31: goto F90e4; c9fbd: Cd106: goto d0442; b68ee: $hasPermissions = $this->hasPermissions(); goto c4d51; c4d51: if (true === $hasPermissions) { goto A49c1; } goto F36e0; a2f77: $response = $this->render("\106\x72\157\156\x74\145\x6e\144\x2f\x53\151\x74\145\x2f\116\x65\167\x2f\x72\145\166\x65\162\x73\x65\x2d\160\162\x6f\x78\171\56\150\164\x6d\x6c\56\164\167\x69\x67", ["\165\x73\x65\162" => $user, "\146\x6f\162\155" => $form->createView(), "\146\x6f\x72\x6d\x45\x72\x72\157\x72\163" => $this->formErrors]); goto df898; Adae4: $user = $this->getUser(); goto D611a; E5a28: A49c1: goto Adae4; Ecc92: if (!(true === $form->isSubmitted())) { goto E9114; } goto df0f4; df0f4: $response = $this->handleReverseProxyForm($request, $form); goto B0c60; B01cd: if (!(true === $request->isMethod("\x50\117\x53\124"))) { goto Ac91e; } goto acf57; B0c60: if (!(false === is_null($response))) { goto Cd106; } goto cf42d; E751b: Ac91e: goto a2f77; e92db: } private function createReverseProxyForm() : Form { goto b8685; b8685: $form = $this->createForm("\101\x70\x70\134\106\x6f\162\155\x5c\x53\x69\x74\145\x4e\x65\x77\x52\145\166\x65\162\163\145\x50\x72\157\170\x79\x54\x79\x70\145", [], ["\x61\143\x74\151\x6f\156" => $this->generateUrl("\143\x6c\160\x5f\163\x69\164\145\137\x72\145\x76\145\162\x73\145\x5f\160\162\x6f\x78\171\137\x6e\145\167"), "\x6d\145\x74\x68\x6f\144" => "\120\117\x53\x54", "\x61\164\x74\x72" => ["\x69\144" => "\x6e\x65\167\55\162\x65\x76\x65\162\x73\x65\x2d\160\162\x6f\x78\x79\x2d\146\x6f\x72\155"]]); goto dc612; D03b7: return $form; goto be94a; dc612: $form->add("\x73\x75\x62\x6d\151\164", SubmitType::class, ["\141\x74\164\x72" => ["\x63\154\x61\x73\163" => "\142\x74\x6e\x20\142\164\156\x2d\x6c\147\40\x62\x74\x6e\x2d\142\154\x75\x65"], "\154\141\x62\145\x6c" => "\x43\x72\x65\x61\164\x65"]); goto D03b7; be94a: } private function handleReverseProxyForm(Request $request, Form $form) { goto fecd1; fecd1: if (true === $form->isValid()) { goto E1f8b; } goto c1ce0; Bce45: fe0e7: goto a12b0; dec97: try { goto Cbc6d; F455a: return $response; goto de83b; Db86e: $this->createReverseProxySite($domainName, $reverseProxyUrl, $siteUser, $siteUserPassword); goto Bbd35; dfae2: $session->getFlashBag()->set("\163\x75\143\143\x65\163\x73", $this->translator->trans("\x52\x65\x76\x65\162\x73\x65\x20\x50\162\x6f\170\171\40\150\x61\163\x20\x62\x65\x65\156\x20\x63\x72\x65\141\164\145\x64\x2e")); goto b15fb; Bbd35: $eventData = ["\163\x69\x74\x65" => $domainName, "\163\x69\164\x65\x55\x73\145\x72" => $siteUser, "\162\145\x76\145\162\x73\x65\120\162\157\x78\x79\125\x72\154" => $reverseProxyUrl]; goto e0cff; e0cff: EventQueue::addEvent(EventQueue::EVENT_SITE_REVERSE_PROXY_CREATE, $user, $eventData, $request); goto dfae2; D1180: goto B5ce0; goto A2b78; A8eb6: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto f886d; eb1f1: $reverseProxyUrl = $form->get("\x72\x65\x76\145\162\163\x65\120\162\x6f\x78\171\x55\162\x6c")->getData(); goto a0ead; Cbc6d: $session = $request->getSession(); goto f6ba4; b15fb: $response = $this->redirect($this->generateUrl("\x63\x6c\x70\137\x73\151\x74\x65\x73")); goto F455a; f6ba4: $domainName = $form->get("\x64\x6f\x6d\x61\151\x6e\x4e\141\x6d\x65")->getData(); goto Acad3; f886d: if (false === is_null($vhostTemplateEntity)) { goto Fb304; } goto d31cd; a0ead: $vhostTemplateName = self::REVERSE_PROXY_VHOST_TEMPLATE_NAME; goto A8eb6; de83b: B5ce0: goto Ea6cf; A1929: $siteUserPassword = $form->get("\x73\x69\x74\x65\x55\x73\145\x72\x50\x61\163\x73\167\157\x72\x64")->getData(); goto eb1f1; c900c: $user = $this->getUser(); goto Db86e; A2b78: Fb304: goto c900c; d31cd: throw new \Exception(sprintf("\126\150\157\163\164\40\164\x65\x6d\x70\x6c\141\x74\145\x20\x22\45\x73\42\40\156\x6f\164\40\x66\x6f\x75\156\144\x2e", $vhostTemplateName)); goto D1180; Acad3: $siteUser = $form->get("\163\151\x74\145\125\163\145\x72")->getData(); goto A1929; Ea6cf: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\144\x61\156\x67\145\x72", $this->translator->trans("\101\156\40\x65\162\162\157\x72\x20\x68\x61\x73\40\x6f\143\143\x75\162\x72\x65\144\54\x20\x65\162\162\x6f\x72\40\155\x65\163\163\141\x67\x65\x3a\x20\45\145\x72\x72\157\162\x4d\x65\163\163\x61\x67\145\x25", ["\45\145\x72\162\157\x72\115\145\x73\x73\141\x67\145\45" => $e->getMessage()])); } goto Bce45; Fb3f5: goto fe0e7; goto da519; da519: E1f8b: goto dec97; c1ce0: $this->formErrors = $this->getErrorMessages($form); goto Fb3f5; a12b0: } public function newNodejsSite(Request $request) : Response { goto a10c4; ba4bf: if (true === $hasPermissions) { goto a2397; } goto Bd9e0; Ee243: $form->handleRequest($request); goto a67ad; Fbb4b: F2437: goto e317f; ab5af: $response = $this->handleCreateNodejsSiteForm($request, $form); goto e3105; Bd9e0: $response = $this->redirect($this->generateUrl("\x63\154\160\137\x73\151\164\145\163")); goto D385f; c9dd1: f68bf: goto e4d46; Ee939: b136f: goto c9dd1; e3105: if (!(false === is_null($response))) { goto E28f4; } goto Fd89f; Fd89f: return $response; goto d7349; e4d46: $response = $this->render("\x46\x72\x6f\x6e\164\x65\x6e\x64\57\123\x69\164\x65\57\116\145\x77\x2f\156\157\144\145\152\163\x2e\x68\x74\x6d\x6c\56\x74\167\x69\x67", ["\165\163\145\x72" => $user, "\146\157\162\x6d" => $form->createView(), "\146\157\x72\x6d\105\x72\x72\157\162\x73" => $this->formErrors]); goto Fbb4b; Aa9a0: $form = $this->createNodejsSiteForm($nodejsSettings); goto E6e9b; E6e9b: if (!(true === $request->isMethod("\x50\117\123\124"))) { goto f68bf; } goto Ee243; a67ad: if (!(true === $form->isSubmitted())) { goto b136f; } goto ab5af; a10c4: $hasPermissions = $this->hasPermissions(); goto ba4bf; f2c3c: $nodejsSettings = new NodejsSettingsEntity(); goto Aa9a0; A1124: a2397: goto Afef1; Afef1: $user = $this->getUser(); goto f2c3c; D385f: goto F2437; goto A1124; e317f: return $response; goto C3519; d7349: E28f4: goto Ee939; C3519: } private function createNodejsSiteForm(NodejsSettingsEntity $nodejsSettings) : Form { goto C5033; B3b88: $form->add("\x73\x75\x62\x6d\151\x74", SubmitType::class, ["\141\x74\164\x72" => ["\143\x6c\141\x73\x73" => "\x62\164\156\x20\x62\x74\156\x2d\154\147\40\x62\164\x6e\55\x62\154\x75\x65"], "\154\x61\142\145\x6c" => "\103\x72\x65\x61\x74\145"]); goto Eb0b6; C5033: $form = $this->createForm("\x41\160\160\x5c\106\x6f\x72\x6d\134\123\x69\x74\145\116\x65\167\116\157\x64\x65\152\163\x54\171\x70\x65", $nodejsSettings, ["\x61\143\164\151\x6f\156" => $this->generateUrl("\143\x6c\x70\x5f\163\151\164\145\x5f\x6e\x6f\x64\x65\152\163\x5f\x6e\145\x77"), "\x6d\145\164\150\157\144" => "\x50\x4f\x53\124", "\x61\164\x74\x72" => ["\151\144" => "\x6e\145\167\x2d\x6e\157\144\145\x6a\x73\55\x73\x69\164\145\x2d\x66\157\162\155"]]); goto B3b88; Eb0b6: return $form; goto F2ebd; F2ebd: } private function handleCreateNodejsSiteForm(Request $request, Form $form) { goto Feab6; ea468: try { goto dd108; Ae092: $response = $this->redirect($this->generateUrl("\x63\154\x70\x5f\x73\151\x74\x65\163")); goto D1136; F7afa: A26ed: goto e99b5; dd108: $session = $request->getSession(); goto c4164; Ac66b: $siteUser = $form->get("\163\x69\x74\145\125\x73\x65\162")->getData(); goto F33f7; Be609: if (false === is_null($vhostTemplateEntity)) { goto A26ed; } goto Ad3ce; D1347: $port = $form->get("\x70\x6f\x72\164")->getData(); goto Ac66b; d8adb: EventQueue::addEvent(EventQueue::EVENT_SITE_NODEJS_CREATE, $user, $eventData, $request); goto a8751; d30f2: $this->createNodejsSite($domainName, $nodejsVersion, $port, $siteUser, $siteUserPassword); goto a33ec; e99b5: $user = $this->getUser(); goto d30f2; feff4: A75f0: goto e8d0f; feb83: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto Be609; db937: $vhostTemplateName = self::NODEJS_VHOST_TEMPLATE_NAME; goto feb83; a33ec: $eventData = ["\163\151\x74\145" => $domainName, "\x73\151\164\x65\x55\x73\145\x72" => $siteUser, "\156\157\144\145\152\163\x56\x65\162\x73\151\x6f\156" => $nodejsVersion, "\160\x6f\x72\164" => $port]; goto d8adb; c4164: $domainName = $form->get("\x64\157\x6d\x61\x69\x6e\x4e\141\155\x65")->getData(); goto D7924; D1136: return $response; goto feff4; D7924: $nodejsVersion = $form->get("\x6e\x6f\x64\145\152\x73\x56\145\x72\x73\x69\157\x6e")->getData(); goto D1347; de1d8: goto A75f0; goto F7afa; a8751: $session->getFlashBag()->set("\163\x75\x63\143\x65\163\163", $this->translator->trans("\123\151\164\145\40\x68\141\163\x20\x62\145\x65\156\40\143\x72\145\141\x74\x65\x64\56")); goto Ae092; F33f7: $siteUserPassword = $form->get("\163\151\164\145\125\x73\x65\162\x50\x61\163\x73\167\x6f\x72\x64")->getData(); goto db937; Ad3ce: throw new \Exception(sprintf("\x56\150\x6f\x73\x74\40\164\145\155\x70\154\x61\164\145\40\42\x25\x73\42\x20\156\x6f\164\40\x66\157\x75\x6e\144\56", $vhostTemplateName)); goto de1d8; e8d0f: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\144\x61\156\147\145\162", $this->translator->trans("\101\x6e\40\145\x72\x72\157\x72\40\150\141\x73\x20\157\143\x63\165\x72\162\x65\x64\54\x20\x65\x72\162\x6f\x72\x20\155\x65\163\163\x61\x67\x65\x3a\x20\45\x65\x72\x72\157\x72\x4d\145\x73\163\141\x67\x65\x25", ["\x25\145\x72\x72\x6f\162\115\145\x73\163\141\147\145\x25" => $e->getMessage()])); } goto e1f72; bc813: goto aaf5e; goto A0d1b; Feab6: if (true === $form->isValid()) { goto Be989; } goto d2e72; d2e72: $this->formErrors = $this->getErrorMessages($form); goto bc813; A0d1b: Be989: goto ea468; e1f72: aaf5e: goto bb2b8; bb2b8: } public function newStaticSite(Request $request) : Response { goto fccd7; A61f7: ad070: goto Bbdd3; Be139: return $response; goto A61f7; c3048: D43e8: goto A4ae4; fccd7: $hasPermissions = $this->hasPermissions(); goto B274c; Eb785: $response = $this->handleCreateStaticSiteForm($request, $form); goto Fd654; A5b4c: $form->handleRequest($request); goto be31e; B274c: if (true === $hasPermissions) { goto F197c; } goto e9361; A7b80: F197c: goto f45a0; A4ae4: return $response; goto D5322; d6021: ebc80: goto b7ef5; B681b: if (!(true === $request->isMethod("\x50\117\123\124"))) { goto ebc80; } goto A5b4c; Fd654: if (!(false === is_null($response))) { goto ad070; } goto Be139; e9361: $response = $this->redirect($this->generateUrl("\143\x6c\160\137\163\x69\x74\x65\163")); goto Be7c7; Be7c7: goto D43e8; goto A7b80; be31e: if (!(true === $form->isSubmitted())) { goto e4f1c; } goto Eb785; bcb8a: $form = $this->createStaticSiteForm(); goto B681b; f45a0: $user = $this->getUser(); goto bcb8a; b7ef5: $response = $this->render("\x46\x72\157\x6e\164\x65\156\144\57\x53\x69\164\x65\x2f\x4e\x65\x77\57\x73\x74\x61\x74\151\143\56\150\x74\155\154\56\164\167\151\147", ["\165\163\x65\x72" => $user, "\146\x6f\162\x6d" => $form->createView(), "\146\157\162\155\105\162\x72\157\x72\x73" => $this->formErrors]); goto c3048; Bbdd3: e4f1c: goto d6021; D5322: } private function createStaticSiteForm() : Form { goto ef2ff; ef2ff: $form = $this->createForm("\x41\160\160\134\106\x6f\x72\155\x5c\x53\151\164\x65\x4e\145\167\x53\x74\x61\164\x69\x63\x54\171\x70\x65", [], ["\141\x63\164\151\157\156" => $this->generateUrl("\143\x6c\x70\137\163\x69\x74\145\137\163\164\141\x74\x69\143\137\156\145\167"), "\155\x65\x74\x68\x6f\144" => "\x50\117\x53\124", "\x61\x74\x74\162" => []]); goto C4744; C4744: $form->add("\163\x75\x62\155\x69\x74", SubmitType::class, ["\x61\x74\x74\162" => ["\x63\154\x61\x73\x73" => "\142\x74\156\x20\142\x74\156\x2d\154\147\x20\x62\x74\156\55\x62\154\x75\145"], "\x6c\x61\x62\145\x6c" => "\103\x72\x65\x61\x74\145"]); goto E1eb7; E1eb7: return $form; goto A13cc; A13cc: } private function handleCreateStaticSiteForm(Request $request, Form $form) { goto A9cf3; ceeb9: goto B5e36; goto f312e; De044: B5e36: goto b6edd; f312e: D74b6: goto d72c7; A9cf3: if (true === $form->isValid()) { goto D74b6; } goto Acacb; d72c7: try { goto c0f29; f0dfc: $this->createStaticSite($domainName, $siteUser, $siteUserPassword); goto e645e; D9482: $response = $this->redirect($this->generateUrl("\143\154\x70\137\163\x69\164\145\x73")); goto Fe647; C3c6c: EventQueue::addEvent(EventQueue::EVENT_SITE_STATIC_CREATE, $user, $eventData, $request); goto ed510; D3bbb: if (false === is_null($vhostTemplateEntity)) { goto F447a; } goto e94c1; dcec3: $domainName = $form->get("\144\x6f\155\141\x69\156\x4e\x61\x6d\x65")->getData(); goto d91da; ed510: $session->getFlashBag()->set("\x73\x75\x63\143\145\x73\x73", $this->translator->trans("\123\x69\164\x65\40\x68\141\x73\40\x62\145\145\156\x20\143\x72\x65\x61\164\x65\x64\x2e")); goto D9482; Fe647: return $response; goto afe43; d91da: $siteUser = $form->get("\x73\x69\164\145\x55\163\x65\162")->getData(); goto D33f9; F25ad: $user = $this->getUser(); goto f0dfc; e94c1: throw new \Exception(sprintf("\x56\150\x6f\163\x74\40\164\x65\155\x70\154\141\x74\x65\40\x22\x25\x73\x22\40\x6e\x6f\x74\x20\146\157\x75\156\144\x2e", $vhostTemplateName)); goto Eee2b; Eee2b: goto Abe31; goto b3ace; c0f29: $session = $request->getSession(); goto dcec3; D33f9: $siteUserPassword = $form->get("\x73\x69\164\x65\x55\163\x65\162\120\141\163\163\x77\157\x72\x64")->getData(); goto e85c2; b3ace: F447a: goto F25ad; e645e: $eventData = ["\163\x69\x74\x65" => $domainName, "\x73\x69\x74\145\125\163\145\x72" => $siteUser]; goto C3c6c; afe43: Abe31: goto d4bc7; d12f1: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto D3bbb; e85c2: $vhostTemplateName = self::STATIC_SITE_VHOST_TEMPLATE_NAME; goto d12f1; d4bc7: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\141\x6e\x67\x65\x72", $this->translator->trans("\x41\x6e\x20\x65\x72\162\157\162\40\150\141\163\40\x6f\x63\143\165\x72\162\145\144\54\40\145\162\162\157\x72\40\x6d\145\163\163\141\x67\145\x3a\40\x25\x65\x72\x72\157\x72\115\x65\163\163\x61\147\145\x25", ["\x25\145\x72\162\x6f\162\x4d\x65\163\x73\141\147\145\x25" => $e->getMessage()])); } goto De044; Acacb: $this->formErrors = $this->getErrorMessages($form); goto ceeb9; b6edd: } public function newPhpSite(Request $request) : Response { goto f9ddd; e7f37: ee9c2: goto bd22a; E9ad2: goto b5984; goto f14a4; ca0cc: return $response; goto af3a8; Af312: if (!(false === is_null($response))) { goto F0522; } goto C9b16; A271d: $form = $this->createPhpSiteForm(); goto d84b8; bd22a: $response = $this->render("\x46\x72\157\156\164\x65\x6e\x64\x2f\123\151\164\145\57\116\x65\167\x2f\x70\150\160\56\150\164\x6d\x6c\56\x74\x77\151\x67", ["\165\163\145\162" => $user, "\x66\x6f\162\155" => $form->createView(), "\146\157\x72\155\x45\x72\162\x6f\162\163" => $this->formErrors]); goto A57f8; ae183: Dba49: goto e7f37; a790c: F0522: goto ae183; Fb06d: if (!(true === $form->isSubmitted())) { goto Dba49; } goto Ef0cb; e928c: $user = $this->getUser(); goto A271d; C9b16: return $response; goto a790c; D9480: if (true === $hasPermissions) { goto Ec42c; } goto e1d47; d84b8: if (!(true === $request->isMethod("\120\117\x53\x54"))) { goto ee9c2; } goto Af593; Af593: $form->handleRequest($request); goto Fb06d; f14a4: Ec42c: goto e928c; Ef0cb: $response = $this->handleCreatePhpSiteForm($request, $form); goto Af312; e1d47: $response = $this->redirect($this->generateUrl("\x63\x6c\160\137\x73\x69\164\145\x73")); goto E9ad2; f9ddd: $hasPermissions = $this->hasPermissions(); goto D9480; A57f8: b5984: goto ca0cc; af3a8: } private function createPhpSiteForm() : Form { goto d23bd; e7552: $form->add("\x73\165\x62\x6d\x69\x74", SubmitType::class, ["\141\164\x74\x72" => ["\143\x6c\x61\163\163" => "\142\x74\156\x20\x62\x74\x6e\x2d\x6c\147\x20\x62\164\156\55\x62\154\x75\x65"], "\154\141\142\145\x6c" => "\103\x72\145\x61\x74\x65"]); goto a994e; a994e: return $form; goto D64d2; d23bd: $form = $this->createForm("\x41\160\160\134\x46\157\162\155\134\x53\x69\164\x65\x4e\145\167\120\x68\x70\x54\171\160\x65", [], ["\x61\143\164\151\x6f\x6e" => $this->generateUrl("\143\154\160\137\163\151\164\x65\137\160\150\x70\137\x6e\145\167"), "\x6d\145\164\150\157\x64" => "\x50\x4f\x53\x54", "\141\164\x74\162" => []]); goto e7552; D64d2: } private function handleCreatePhpSiteForm(Request $request, Form $form) { goto e0862; b195b: da251: goto f08a3; Ffb68: $this->formErrors = $this->getErrorMessages($form); goto e3a4b; Bfdb6: try { goto Fefe2; D5a10: if (false === is_null($vhostTemplateEntity)) { goto e0ef8; } goto Ce43e; Fcae3: EventQueue::addEvent(EventQueue::EVENT_SITE_PHP_CREATE, $user, $eventData, $request); goto Ba7a2; fb86f: $vhostTemplateName = $form->get("\x61\160\160\154\151\x63\141\164\x69\x6f\x6e")->getData(); goto a5870; A7bbd: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto D5a10; e9089: e0ef8: goto F9d62; B1d78: $rootDirectory = sprintf("\45\163\x2f\45\163", $rootDirectory, ltrim(rtrim($vhostTemplateEntity->getRootDirectory(), "\x2f"), "\57")); goto Eef79; D18c6: if (!(false === empty($vhostTemplateEntity->getVarnishCacheSettings()))) { goto af872; } goto F26ba; a5870: $domainName = $form->get("\x64\x6f\155\141\151\156\x4e\141\x6d\145")->getData(); goto d9128; F9d62: $user = $this->getUser(); goto b0e66; b2c78: D1e80: goto Fe30c; A2c76: $eventData = ["\163\151\164\145" => $domainName, "\x76\x68\x6f\x73\x74\124\x65\x6d\x70\x6c\x61\x74\145\x4e\x61\x6d\x65" => $vhostTemplateName, "\160\x68\x70\x56\145\x72\163\x69\157\156" => $phpVersion, "\163\151\x74\145\x55\x73\145\162" => $siteUser]; goto Fcae3; D90d0: af872: goto Eef35; Fa2d9: return $response; goto b2c78; Ba7a2: $session->getFlashBag()->set("\163\x75\143\143\x65\x73\x73", $this->translator->trans("\x53\151\x74\x65\x20\x68\141\x73\x20\142\x65\x65\156\x20\143\162\x65\141\x74\145\144\x2e")); goto da255; C754e: if (!(false === empty($vhostTemplateEntity->getRootDirectory()))) { goto d9096; } goto B1d78; Fefe2: $session = $request->getSession(); goto fb86f; Da9cb: goto D1e80; goto e9089; b0e66: $rootDirectory = $domainName; goto C754e; d9128: $phpVersion = $form->get("\x70\150\160\x56\145\x72\163\151\157\x6e")->getData(); goto f685b; da255: $response = $this->redirect($this->generateUrl("\x63\154\160\137\163\151\164\x65\163")); goto Fa2d9; b6836: $varnishCacheSettings = []; goto D18c6; c1fa4: $siteUserPassword = $form->get("\163\151\164\x65\125\x73\145\x72\120\x61\163\163\167\x6f\x72\x64")->getData(); goto A7bbd; f685b: $siteUser = $form->get("\163\x69\x74\x65\x55\163\x65\x72")->getData(); goto c1fa4; Eef79: d9096: goto b6836; F26ba: $varnishCacheSettings = (array) json_decode($vhostTemplateEntity->getVarnishCacheSettings(), true); goto D90d0; Eef35: $this->createPhpSite($domainName, $rootDirectory, $phpVersion, $vhostTemplateName, $siteUser, $siteUserPassword, $varnishCacheSettings); goto A2c76; Ce43e: throw new \Exception(sprintf("\126\x68\x6f\x73\x74\40\x74\145\x6d\x70\x6c\141\x74\x65\40\x22\45\163\x22\x20\x6e\x6f\164\40\146\x6f\x75\x6e\144\x2e", $vhostTemplateName)); goto Da9cb; Fe30c: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\144\x61\x6e\147\x65\x72", $this->translator->trans("\x41\x6e\40\x65\x72\x72\x6f\162\x20\150\141\x73\40\x6f\x63\143\165\x72\162\x65\x64\x2c\x20\145\x72\162\157\162\x20\x6d\x65\163\163\x61\x67\145\72\40\45\x65\x72\x72\157\x72\115\x65\x73\163\x61\x67\145\x25", ["\45\x65\162\x72\x6f\x72\x4d\145\163\x73\x61\x67\x65\x25" => $e->getMessage()])); } goto b195b; ec84e: c19b0: goto Bfdb6; e3a4b: goto da251; goto ec84e; e0862: if (true === $form->isValid()) { goto c19b0; } goto Ffb68; f08a3: } public function newWordPressSite(Request $request) : Response { goto Bbcdf; bc7a7: $form = $this->createWordPressSiteForm(); goto d414d; Bd134: return $response; goto d8325; d4d11: goto af6ce; goto f7390; Ccd13: f3ed6: goto bee1e; D5b9f: $response = $this->render("\106\x72\157\156\164\145\x6e\x64\57\123\x69\x74\145\x2f\116\145\167\57\167\157\x72\144\160\x72\x65\163\163\56\150\x74\x6d\154\x2e\164\167\x69\x67", ["\165\163\x65\x72" => $user, "\x66\157\x72\x6d" => $form->createView(), "\146\x6f\162\155\x45\162\x72\157\162\x73" => $this->formErrors]); goto C1bfa; Cbcf9: $form->handleRequest($request); goto Fee39; c2ba9: D3f03: goto Ccd13; Fee39: if (!(true === $form->isSubmitted())) { goto f3ed6; } goto d4558; Bbcdf: $hasPermissions = $this->hasPermissions(); goto Fd237; d414d: if (!(true === $request->isMethod("\120\117\x53\x54"))) { goto e00e5; } goto Cbcf9; Bce5a: $response = $this->redirect($this->generateUrl("\x63\154\160\137\163\151\x74\x65\163")); goto d4d11; C1bfa: af6ce: goto Bd134; bee1e: e00e5: goto D5b9f; C6a6d: return $response; goto c2ba9; Fd237: if (true === $hasPermissions) { goto Fd0e3; } goto Bce5a; d4558: $response = $this->handleCreateWordPressSiteForm($request, $form); goto A347f; A347f: if (!(false === is_null($response))) { goto D3f03; } goto C6a6d; f7390: Fd0e3: goto b8a1f; b8a1f: $user = $this->getUser(); goto bc7a7; d8325: } private function createWordPressSiteForm() : Form { goto D45bf; a6623: return $form; goto E3cf2; D45bf: $form = $this->createForm("\x41\160\160\134\106\x6f\x72\155\x5c\x53\x69\164\x65\x4e\145\167\x57\x6f\162\144\x50\x72\x65\163\x73\x54\x79\x70\145", [], ["\141\143\164\x69\x6f\156" => $this->generateUrl("\x63\154\160\x5f\163\151\x74\x65\x5f\167\157\x72\x64\160\x72\x65\x73\163\x5f\156\145\167"), "\155\145\x74\150\157\x64" => "\120\117\x53\124", "\x61\x74\x74\162" => ["\151\144" => "\156\x65\167\x2d\167\x6f\x72\144\x70\162\145\163\x73\x2d\163\x69\164\x65\x2d\x66\x6f\x72\x6d"]]); goto b8807; b8807: $form->add("\163\165\142\155\151\x74", SubmitType::class, ["\x61\164\164\162" => ["\143\x6c\141\163\x73" => "\142\164\x6e\x20\142\164\x6e\x2d\x6c\x67\40\x62\x74\156\x2d\142\154\165\x65"], "\154\x61\142\x65\x6c" => "\x43\162\x65\141\x74\x65"]); goto a6623; E3cf2: } private function handleCreateWordPressSiteForm(Request $request, Form $form) { goto D8cfa; Cd075: try { goto C09e8; f5139: $siteUserPassword = $form->get("\x73\151\x74\145\x55\163\x65\x72\x50\141\x73\x73\167\x6f\x72\144")->getData(); goto f2e2b; Ab7be: $domainName = $form->get("\x64\157\155\x61\151\156\116\x61\155\x65")->getData(); goto A801c; badde: $databaseManager = new DatabaseManager($activeDatabaseServerEntity); goto e15f8; F9a34: $databaseEntity->setDatabaseServer($activeDatabaseServerEntity); goto B73a0; B73a0: $databaseEntity->setName($databaseName); goto D2857; D30a8: $url = sprintf("\x68\164\x74\160\x73\72\x2f\57\45\163", $siteEntity->getDomainName()); goto c7bc5; be332: $response = $this->redirect($this->generateUrl("\143\154\160\x5f\x73\x69\164\x65\x5f\167\x6f\162\144\160\162\x65\x73\x73\137\x69\156\163\x74\141\154\154\145\144")); goto d1f78; e41c5: $siteCredentials[$this->translator->trans("\104\141\x74\141\x62\x61\x73\145")] = [$this->translator->trans("\x48\157\x73\164") => $activeDatabaseServerEntity->getHost(), $this->translator->trans("\120\x6f\162\164") => $activeDatabaseServerEntity->getPort(), $this->translator->trans("\x44\141\x74\x61\142\141\x73\145\40\116\x61\x6d\145") => $databaseName, $this->translator->trans("\x44\141\x74\141\x62\141\163\x65\40\125\x73\x65\x72\40\116\141\155\x65") => $databaseUserName, $this->translator->trans("\x44\141\164\141\x62\x61\163\x65\x20\125\x73\x65\162\x20\x50\x61\163\x73\167\x6f\162\144") => $databaseUserPassword]; goto fda53; Bfe10: $databaseUserEntity->setUserName($databaseUserName); goto D8b7d; F589b: $databaseUserName = $this->getUniqueDatabaseUserName($generatedName); goto Af41d; f6c99: $siteUser = $form->get("\163\151\164\x65\125\163\145\162")->getData(); goto f5139; D8b7d: $databaseUserEntity->setPassword(Crypto::encrypt($databaseUserPassword)); goto f4910; d3f6a: $adminUserName = $form->get("\x61\x64\x6d\x69\x6e\x55\163\x65\162\116\141\155\145")->getData(); goto Cb130; Bd1bb: $varnishCacheSettings = (array) json_decode($vhostTemplateEntity->getVarnishCacheSettings(), true); goto a82d9; c7e02: if (!(false === empty($vhostTemplateEntity->getVarnishCacheSettings()))) { goto C8b50; } goto Bd1bb; B5a27: $databaseManager->createUser($databaseUserEntity); goto B53e7; C09e8: $user = $this->getUser(); goto a27bd; E4deb: $databaseEntity = $this->databaseEntityManager->createEntity(); goto F9a34; A801c: $generatedName = $this->generateNameFromDomainName($domainName); goto f6c99; d1370: $isMultiSite = $form->get("\x6d\165\x6c\164\151\123\151\164\145")->getData(); goto adfdb; f2e2b: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::WORDPRESS_VHOST_TEMPLATE_NAME); goto Ffca0; B53e7: $this->siteEntityManager->updateEntity($siteEntity); goto dbde3; D2857: $databaseEntity->setSite($siteEntity); goto d01b5; e917c: $eventData = ["\x73\151\x74\x65" => $domainName, "\163\x69\164\145\x55\163\145\162" => $siteUser]; goto c03b6; c7bc5: $title = $form->get("\163\151\x74\145\x54\151\x74\154\x65")->getData(); goto d3f6a; e15f8: $databaseManager->createDatabase($databaseEntity); goto B5a27; cd3fa: $databaseName = $this->getUniqueDatabaseName($generatedName); goto F589b; F9370: $instance = $request->attributes->get("\x69\156\163\164\x61\x6e\x63\x65"); goto ee028; Ffca0: $phpVersion = $vhostTemplateEntity->getPhpVersion(); goto Ed016; a82d9: C8b50: goto Df488; bdf50: $databaseUserEntity->setDatabase($databaseEntity); goto Fabb3; Fabb3: $databaseEntity->addUser($databaseUserEntity); goto B00ba; d01b5: $databaseUserEntity = $this->databaseUserEntityManager->createEntity(); goto Bfe10; F6588: if (!(false === is_null($siteEntity))) { goto C6f95; } goto cd3fa; adfdb: $activeDatabaseServerEntity = $this->databaseServerEntityManager->getActiveDatabaseServer(); goto E4deb; fda53: $siteCredentials["\x57\157\x72\144\120\x72\145\x73\x73"] = [$this->translator->trans("\101\x64\155\x69\x6e\40\x45\55\115\x61\151\x6c") => $adminEmail, $this->translator->trans("\101\x64\x6d\x69\x6e\40\125\x73\x65\162\x20\116\141\x6d\x65") => $adminUserName, $this->translator->trans("\x41\x64\155\151\156\40\x50\141\x73\163\167\157\162\144") => $adminPassword, $this->translator->trans("\x41\x64\x6d\151\156\x20\x55\162\154") => sprintf("\150\164\x74\x70\x73\x3a\x2f\57\x25\163\57\167\160\55\141\x64\155\151\x6e\57", rtrim($domainName, "\57"))]; goto A3e37; Ed016: $varnishCacheSettings = []; goto c7e02; Af41d: $databaseUserPassword = PasswordGenerator::generate(self::PASSWORD_LENGTH); goto D30a8; d1f78: return $response; goto f4dc2; f5886: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto F9370; c03b6: EventQueue::addEvent(EventQueue::EVENT_SITE_WORDPRESS_CREATE, $user, $eventData, $request); goto be332; Df488: $siteEntity = $this->createPhpSite($domainName, $domainName, $phpVersion, self::WORDPRESS_VHOST_TEMPLATE_NAME, $siteUser, $siteUserPassword, $varnishCacheSettings); goto f5886; f4910: $databaseUserEntity->setPermissions(DatabaseUserEntity::PERMISSIONS_READ_WRITE); goto bdf50; B00ba: $siteEntity->addDatabase($databaseEntity); goto badde; b0b53: $session->set("\x73\x69\x74\x65\x43\162\x65\x64\x65\x6e\164\151\x61\x6c\163", $siteCredentials); goto e917c; cc2c8: $adminEmail = $form->get("\x61\x64\x6d\151\x6e\105\155\141\151\x6c")->getData(); goto d1370; Cb130: $adminPassword = $form->get("\x61\144\155\x69\156\x50\141\163\x73\x77\157\x72\x64")->getData(); goto cc2c8; dbde3: $this->installWordPress($isMultiSite, $url, $title, $adminUserName, $adminPassword, $adminEmail, $siteEntity, $activeDatabaseServerEntity, $databaseEntity, $databaseUserEntity); goto e41c5; A3e37: C6f95: goto b0b53; a27bd: $session = $request->getSession(); goto Ab7be; ee028: $siteCredentials = [$this->translator->trans("\x53\x69\164\x65") => [$this->translator->trans("\111\x50\x20\101\144\x64\162\x65\x73\x73") => $instance->getIpv4PublicIp(), $this->translator->trans("\x44\157\x6d\x61\151\156\40\116\141\x6d\x65") => sprintf("\x68\x74\x74\x70\x73\72\57\57\45\x73", $domainName), $this->translator->trans("\x53\151\164\145\40\x55\163\x65\162") => $siteUser, $this->translator->trans("\x50\x61\x73\163\167\x6f\162\x64") => $siteUserPassword]]; goto F6588; f4dc2: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\141\156\x67\145\x72", $this->translator->trans("\x41\156\40\x65\x72\162\157\162\x20\x68\141\163\40\x6f\143\x63\x75\x72\162\145\x64\x2c\x20\x65\x72\162\x6f\x72\40\x6d\x65\x73\x73\x61\147\x65\x3a\x20\x25\145\x72\x72\157\162\115\x65\x73\163\x61\x67\145\x25", ["\x25\145\x72\x72\157\162\115\145\163\x73\x61\147\x65\45" => $e->getMessage()])); } goto d027c; D8cfa: if (true === $form->isValid()) { goto Bd1b0; } goto d487b; d027c: f89d4: goto Bebf8; F6662: goto f89d4; goto Acb63; Acb63: Bd1b0: goto Cd075; d487b: $this->formErrors = $this->getErrorMessages($form); goto F6662; Bebf8: } private function installWordPress(bool $isMultiSite, string $url, string $title, string $adminUserName, string $adminPassword, string $adminEmail, SiteEntity $siteEntity, DatabaseServerEntity $databaseServerEntity, DatabaseEntity $databaseEntity, DatabaseUserEntity $databaseUserEntity) : void { goto A96ac; A96ac: $databaseHost = sprintf("\x25\163\72\45\x73", $databaseServerEntity->getHost(), $databaseServerEntity->getPort()); goto c71fc; E0875: Beaee: goto Cfba7; Bee50: if (!(true === $isMultiSite)) { goto C76a9; } goto Bf61c; Cfba7: C76a9: goto A3f62; e5a9b: a0fb6: goto Bee50; Bf61c: $this->wordPressMultiSiteConfigValues["\104\117\115\101\x49\x4e\x5f\103\125\122\122\105\x4e\124\x5f\123\111\124\x45"]["\166\x61\154\165\145"] = $siteEntity->getDomainName(); goto D912d; Ab00d: foreach ($this->wordPressDefaultConfigValues as $key => $config) { $wordPressInstaller->setConfigValue($key, $config["\166\x61\154\x75\145"], $config["\162\141\x77"]); E65a0: } goto e5a9b; f9e09: $wordPressInstaller = new WordPressInstaller($siteEntity); goto b9e7c; c71fc: $databaseName = $databaseEntity->getName(); goto A3f54; Bf313: $wordPressInstaller->createConfig($databaseHost, $databaseName, $databaseUserName, $databaseUserPassword); goto A902e; A3f62: $wordPressInstaller->resetPermissions(); goto d917f; D912d: foreach ($this->wordPressMultiSiteConfigValues as $key => $config) { $wordPressInstaller->setConfigValue($key, $config["\x76\x61\x6c\165\x65"], $config["\162\141\x77"]); F806f: } goto E0875; A902e: $wordPressInstaller->installCore($isMultiSite, $url, $title, $adminUserName, $adminPassword, $adminEmail); goto Ab00d; a0960: $databaseUserPassword = Crypto::decrypt($databaseUserEntity->getPassword()); goto f9e09; b9e7c: $wordPressInstaller->downloadAndExtractLatestVersion(); goto Bf313; A3f54: $databaseUserName = $databaseUserEntity->getUserName(); goto a0960; d917f: } private function createPhpSite(string $domainName, string $rootDirectory, string $phpVersion, string $vhostTemplateName, string $siteUser, string $siteUserPassword, array $varnishCacheSettings = []) : SiteEntity { goto C8cd2; f4405: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\56\57\56\x2e\57\56\56\57\162\145\163\x6f\x75\162\143\x65\163\57\156\x67\x69\156\x78\x2f\x76\150\157\x73\164\137\x74\x65\155\160\154\x61\164\x65\57\162\x65\144\x69\x72\x65\143\x74\x2d\150\164\164\x70\63"); goto a27c9; C9e40: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto D2061; f10e0: ab954: goto ecc9e; b922a: $phpSite->setVarnishCache(true); goto e2eaf; b17d0: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto dfe72; d079b: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto cf2e3; facfc: $siteEntity->setUserPassword($siteUserPassword); goto e5ccf; Cf6f7: $siteEntity->setVhostTemplate($vhostTemplate); goto c90df; ead0b: $constraintViolationList->addAll($phpSettingsConstraints); goto C9e40; ecbdc: $phpSite->setDomainName($domainName); goto f7e86; db643: $rsaKeyGenerator = new RsaKeyGenerator(); goto c4f17; e9417: $phpSiteCreator->createIndexPhp(); goto Bbdb9; A759e: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto Def9c; be5b9: $certificateEntity->setCsr($csr); goto F28c4; c90df: $phpSettingsEntity = new PhpSettingsEntity(); goto E57aa; Fd864: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto be5b9; fda7c: $subdomain = false === empty($subdomain) ? $subdomain : null; goto d1ef4; Eaf93: Cdc94: goto f4405; ed987: $siteEntity->setCertificate($certificateEntity); goto Fcf4b; f837d: $phpSite->setPhpSettings($phpSettingsEntity); goto b828f; C4fc7: $rootDirectory = trim(rtrim(ltrim($rootDirectory, "\x2f")), "\57"); goto cd585; d2645: Cb4e8: goto Ca9ff; E9dd0: $phpSite->setUserPassword($siteEntity->getUserPassword()); goto ecbdc; d60c8: $phpSiteCreator->resetPermissions(); goto ed15f; A9d8f: $phpSite->setSubdomain($subdomain); goto ec847; cb981: if (!(count($siteConstraints) || count($phpSettingsConstraints))) { goto E8f8e; } goto d61bd; Bbdb9: $phpSiteCreator->createPrivateKeyAndCertificate(); goto D566b; Bab8b: ac0aa: goto fea9a; d61bd: $constraintViolationList = new ConstraintViolationList(); goto E5ff7; cd688: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto fc201; d8619: $phpSite->setCertificate($certificateEntity); goto f837d; cd585: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto A759e; E57aa: $siteEntity->setPhpSettings($phpSettingsEntity); goto b06eb; c4f17: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto a1a72; Def9c: $subdomain = $resolvedDomainName->subDomain()->toString(); goto fda7c; F0844: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto C0864; c90f1: $phpSite->setVarnishCacheSettings($varnishCacheSettings); goto E3bf0; ec10b: $phpSiteCreator->createRootDirectory(); goto d6816; Edbcf: $vhostTemplate->resetProcessors(); goto F0844; C6e36: $this->siteEntityManager->updateEntity($siteEntity); goto c8b6b; d2fcc: if (!(true === $varnishCache)) { goto fc952; } goto b922a; e5ccf: $siteEntity->setRootDirectory($rootDirectory); goto abd86; d912e: goto A5dab; goto Eaf93; C1266: $certificateEntity = $this->certificateEntityManager->createEntity(); goto F9419; Abd4f: $phpSite->setUser($siteEntity->getUser()); goto E9dd0; a26e6: $phpSiteCreator = new PhpSiteCreator($phpSite); goto B75a3; E3b26: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto b30a6; C8cd2: $varnishCache = false === empty($varnishCacheSettings) ? true : false; goto C4fc7; B75a3: $phpSiteCreator->createUser(); goto ec10b; Dd26f: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto Cb953; ec847: $phpSite->setRootDirectory($siteEntity->getRootDirectory()); goto d8619; d146e: $phpSiteCreator->reloadPhpFpmService(); goto d2fcc; b4e63: $subjectAlternativeNames[] = sprintf("\x77\x77\x77\x2e\45\x73", $domainName); goto f10e0; c1372: d13d3: goto F739b; F739b: dcaca: goto Cf6f7; E5ff7: $constraintViolationList->addAll($siteConstraints); goto ead0b; d6816: $phpSiteCreator->createLogrotateFile(); goto e9417; c7b2d: $siteEntity->setDomainName($domainName); goto Ade39; B75bc: $phpSiteCreator->createNginxVhost(); goto Bac3d; c3259: $subjectAlternativeNames[] = $registrableDomain; goto d2645; a27c9: A5dab: goto Dd26f; cf2e3: $certificateEntity->setDefaultCertificate(true); goto Fd864; f0493: $vhostTemplate = sprintf("\45\x73\45\163", $redirectionVhostTemplate, $vhostTemplate); goto c1372; D566b: $phpSiteCreator->createPhpFpmPool(); goto d146e; d2456: $certificateEntity->setCertificate($selfSignedCertificate); goto ed987; Fcf4b: $phpSite = new PhpSite(); goto Abd4f; fea9a: if (!(true === is_null($subdomain) || "\167\167\167" == $subdomain)) { goto dcaca; } goto Aa3de; b828f: $phpSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto a26e6; Aa3de: if (true === isset($_ENV["\101\x50\120\137\x48\124\124\120\x33"]) && "\164\162\x75\x65" === $_ENV["\x41\x50\120\x5f\x48\x54\x54\x50\x33"]) { goto Cdc94; } goto fc7af; e2eaf: $defaultVarnishCacheSettings = ["\x65\x6e\x61\142\x6c\145\x64" => false, "\x73\x65\162\166\x65\162" => self::VARNISH_CACHE_SERVER, "\143\141\x63\150\145\124\141\147\120\x72\145\146\x69\170" => substr(md5(time()), 0, 4)]; goto ae622; F0fae: $siteEntity->setVarnishCache(true); goto Bab8b; abd86: if (!(false === empty($varnishCache))) { goto ac0aa; } goto F0fae; fa0bb: if (!(true === is_null($subdomain))) { goto ab954; } goto b4e63; b30a6: $vhostTemplate->build(); goto b17d0; F9419: $certificateEntity->setSite($siteEntity); goto db643; Cb953: if (!(false === empty($redirectionVhostTemplate))) { goto d13d3; } goto f0493; D2248: $phpSettingsConstraints = $this->validator->validate($phpSettingsEntity); goto cb981; ae622: $varnishCacheSettings = array_merge($defaultVarnishCacheSettings, $varnishCacheSettings); goto f99eb; F28c4: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto d2456; f99eb: $phpSiteCreator->createVarnishCacheStructure($varnishCacheSettings); goto c90f1; C0864: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto E3b26; dfe72: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto C6e36; Bac3d: $phpSiteCreator->reloadNginxService(); goto d60c8; Fd479: $csr = $csrGenerator->generate(); goto d079b; ed15f: $vhostTemplate = new PhpVhostTemplate($phpSite); goto D8a2f; c909e: $siteConstraints = $this->validator->validate($siteEntity); goto D2248; d1ef4: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto cd688; fc201: $siteEntity = $this->siteEntityManager->createEntity(); goto a3292; a3292: $siteEntity->setType(SiteEntity::TYPE_PHP); goto c7b2d; E3bf0: fc952: goto B75bc; D2061: throw new \Exception(implode("\x3b\40", $errorMessages)); goto D3b82; D8a2f: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto Edbcf; Bb8d6: $phpSettingsEntity->setSite($siteEntity); goto c909e; F71fb: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto Fd479; D3b82: E8f8e: goto C1266; b06eb: $phpSettingsEntity->setPhpVersion($phpVersion); goto Bb8d6; Ca9ff: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto F71fb; ecc9e: if (!(false === is_null($subdomain) && "\x77\167\167" == $subdomain)) { goto Cb4e8; } goto c3259; Ade39: $siteEntity->setUser($siteUser); goto facfc; fc7af: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\x2e\x2f\x2e\x2e\57\x2e\x2e\57\162\145\163\x6f\x75\162\143\x65\163\57\x6e\147\151\156\x78\x2f\x76\x68\x6f\x73\x74\x5f\x74\x65\155\160\x6c\141\x74\x65\x2f\x72\145\x64\x69\162\x65\x63\164"); goto d912e; f7e86: $phpSite->setRegistrableDomain($registrableDomain); goto A9d8f; a1a72: $subjectAlternativeNames = []; goto fa0bb; c8b6b: return $siteEntity; goto a2ee1; a2ee1: } private function createNodejsSite(string $domainName, string $nodejsVersion, string $port, string $siteUser, string $siteUserPassword) : SiteEntity { goto caa8e; c6513: $nodejsSite->setCertificate($certificateEntity); goto Cf9b0; f4e80: C3cf0: goto ce6f3; Af1a3: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto b1894; C13be: if (!(true === is_null($subdomain))) { goto A7cae; } goto a838e; e4cc0: A7cae: goto A6200; c9b26: $siteEntity = $this->siteEntityManager->createEntity(); goto ae446; ecc84: $siteEntity->setUserPassword($siteUserPassword); goto ad777; F2de7: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto B2990; C42ae: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto dcf31; B2990: if (!(false === empty($redirectionVhostTemplate))) { goto Cf23a; } goto f24e7; Aa881: $csr = $csrGenerator->generate(); goto Bb8a8; a5631: Cf23a: goto A319f; F8232: $siteEntity->setVhostTemplate($vhostTemplate); goto a4670; ad777: $siteEntity->setRootDirectory($domainName); goto Dbeb0; ca9a0: $nodejsSettingsConstraints = $this->validator->validate($nodejsSettingsEntity); goto F4d1f; Bb222: $rsaKeyGenerator = new RsaKeyGenerator(); goto A1023; e7c5e: $siteConstraints = $this->validator->validate($siteEntity); goto ca9a0; a62c6: $nodejsSettingsEntity->setNodejsVersion($nodejsVersion); goto Dff1d; A319f: d536c: goto F8232; Ff922: $vhostTemplate->build(); goto E0890; E7bd1: $nodejsSiteCreator->createUser(); goto Ef40c; dac14: $nodejsSettingsEntity->setSite($siteEntity); goto e7c5e; cd924: $subjectAlternativeNames[] = $registrableDomain; goto b38b5; a838e: $subjectAlternativeNames[] = sprintf("\x77\x77\167\x2e\x25\163", $domainName); goto e4cc0; Bb8a8: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto eb1bc; df7e8: $nodejsSiteCreator->createNvmDirectory(); goto bed89; a7922: $nodejsSiteCreator->reloadNginxService(); goto fe557; de3cd: $subdomain = $resolvedDomainName->subDomain()->toString(); goto f6aa7; e46c6: $nodejsSite = new NodejsSite(); goto Ecf1a; c9ac0: $constraintViolationList->addAll($nodejsSettingsConstraints); goto ccb26; f6aa7: $subdomain = false === empty($subdomain) ? $subdomain : null; goto E629d; Fadb4: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto f4b87; d7477: $nodejsSiteCreator->createPrivateKeyAndCertificate(); goto f7333; bed89: $nodejsSiteCreator->installNodejs(); goto d7477; acf53: $vhostTemplate = new StaticVhostTemplate($nodejsSite); goto ee7b3; eddb9: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto D4e2d; Bd8ae: $nodejsSite->setUserPassword($siteEntity->getUserPassword()); goto fbaef; E892f: $nodejsSite->setRegistrableDomain($registrableDomain); goto fb31b; A14b4: $vhostTemplate->resetProcessors(); goto C998d; Ef40c: $nodejsSiteCreator->createRootDirectory(); goto Fe946; Adabb: $certificateEntity->setSite($siteEntity); goto Bb222; Cf9b0: $nodejsSite->setNodejsSettings($nodejsSettingsEntity); goto caa36; D2ce2: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto c9b26; C6b20: $siteEntity->setCertificate($certificateEntity); goto e46c6; fbaef: $nodejsSite->setDomainName($domainName); goto E892f; a4670: $nodejsSettingsEntity = new NodejsSettingsEntity(); goto c0090; A0874: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto de3cd; a9de7: $siteEntity->setDomainName($domainName); goto A3125; c0090: $siteEntity->setNodejsSettings($nodejsSettingsEntity); goto a62c6; f24e7: $vhostTemplate = sprintf("\45\x73\x25\x73", $redirectionVhostTemplate, $vhostTemplate); goto a5631; A3125: $siteEntity->setUser($siteUser); goto ecc84; Dff1d: $nodejsSettingsEntity->setPort($port); goto dac14; f4b87: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto Ff922; Cc486: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\56\x2f\56\56\57\56\56\x2f\162\x65\x73\157\165\162\x63\145\x73\57\156\x67\151\156\170\57\166\150\x6f\163\x74\137\x74\145\x6d\160\154\141\164\x65\x2f\162\x65\144\x69\x72\145\143\x74\55\x68\164\x74\160\x33"); goto F2ac1; d2596: goto Ee23b; goto B4516; F2ac1: Ee23b: goto F2de7; ee7b3: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto A14b4; caa8e: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto A0874; dca40: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto cc833; A1023: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto Dbd6a; Dbeb0: if (!(true === is_null($subdomain) || "\167\x77\167" == $subdomain)) { goto d536c; } goto fa959; F5256: return $siteEntity; goto C7d17; E629d: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::NODEJS_VHOST_TEMPLATE_NAME); goto D2ce2; F4d1f: if (!(count($siteConstraints) || count($nodejsSettingsConstraints))) { goto C3cf0; } goto Efbcd; E0890: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto dca40; fa959: if (true === isset($_ENV["\x41\x50\x50\x5f\x48\124\x54\120\63"]) && "\x74\x72\x75\145" === $_ENV["\101\120\120\x5f\110\124\x54\x50\63"]) { goto F20a7; } goto Cb8fb; C998d: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto Fadb4; D4e2d: $certificateEntity->setCertificate($selfSignedCertificate); goto C6b20; dcf31: $certificateEntity->setCsr($csr); goto eddb9; Dbd6a: $subjectAlternativeNames = []; goto C13be; Fe946: $nodejsSiteCreator->createLogrotateFile(); goto df7e8; ae446: $siteEntity->setType(SiteEntity::TYPE_NODEJS); goto a9de7; Cb8fb: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\56\57\56\56\57\56\x2e\57\x72\145\x73\x6f\x75\x72\x63\x65\x73\57\156\147\x69\156\170\57\x76\x68\x6f\163\164\137\164\145\x6d\160\154\141\164\x65\x2f\x72\145\x64\151\x72\x65\x63\164"); goto d2596; ce6f3: $certificateEntity = $this->certificateEntityManager->createEntity(); goto Adabb; fe557: $nodejsSiteCreator->resetPermissions(); goto acf53; caa36: $nodejsSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto b1c67; Ecf1a: $nodejsSite->setUser($siteEntity->getUser()); goto Bd8ae; bc97a: throw new \Exception(implode("\73\40", $errorMessages)); goto f4e80; fb31b: $nodejsSite->setSubdomain($subdomain); goto Ae16f; b38b5: ebf09: goto Af1a3; D0c2a: $constraintViolationList->addAll($siteConstraints); goto c9ac0; f7333: $nodejsSiteCreator->createNginxVhost(); goto a7922; B4516: F20a7: goto Cc486; eb1bc: $certificateEntity->setDefaultCertificate(true); goto C42ae; cc833: $this->siteEntityManager->updateEntity($siteEntity); goto F5256; A6200: if (!(false === is_null($subdomain) && "\x77\x77\x77" == $subdomain)) { goto ebf09; } goto cd924; b1894: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto Aa881; b1c67: $nodejsSiteCreator = new NodejsSiteCreator($nodejsSite); goto E7bd1; Efbcd: $constraintViolationList = new ConstraintViolationList(); goto D0c2a; ccb26: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto bc97a; Ae16f: $nodejsSite->setRootDirectory($siteEntity->getRootDirectory()); goto c6513; C7d17: } private function createPythonSite(string $domainName, string $pythonVersion, string $port, string $siteUser, string $siteUserPassword) : SiteEntity { goto B12a3; Cf738: return $siteEntity; goto Dae84; c633a: $pythonSettingsEntity->setPythonVersion($pythonVersion); goto dc44c; e55df: $siteEntity->setUser($siteUser); goto f0d1e; B019d: F5f8a: goto ea98e; B8568: $vhostTemplate = new PythonVhostTemplate($pythonSite); goto B30f1; E55a3: $constraintViolationList = new ConstraintViolationList(); goto bb789; F9bec: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto Ced6e; bb789: $constraintViolationList->addAll($siteConstraints); goto f77ea; e3e94: $subdomain = false === empty($subdomain) ? $subdomain : null; goto A9ce6; ed589: $pythonSite->setPythonSettings($pythonSettingsEntity); goto dda85; D510a: if (!(true === is_null($subdomain))) { goto a7e05; } goto b98ca; a4381: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto Cf738; b98ca: $subjectAlternativeNames[] = sprintf("\x77\167\167\56\x25\x73", $domainName); goto e123b; eaf3e: $csr = $csrGenerator->generate(); goto Ae576; f65a3: $siteEntity->setRootDirectory($domainName); goto Bed91; F2e07: $pythonSiteCreator->createNginxVhost(); goto d1c5d; e01b0: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\56\x2f\x2e\x2e\x2f\x2e\x2e\x2f\x72\x65\x73\157\165\x72\x63\145\163\x2f\156\147\x69\156\x78\57\166\x68\x6f\x73\x74\x5f\164\x65\x6d\160\x6c\x61\x74\x65\57\162\x65\144\151\x72\x65\x63\164"); goto D5a34; db825: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto eaf3e; C155f: $subdomain = $resolvedDomainName->subDomain()->toString(); goto e3e94; Bb1df: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto E45b6; b6722: E0233: goto Ae41d; ea98e: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto Be0ae; B12a3: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto af812; c17bf: if (!(count($siteConstraints) || count($pythonSettingsConstraints))) { goto af64f; } goto E55a3; E9b8b: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto Cc8a2; Deb8f: E0f3a: goto A9fc9; ecd57: $vhostTemplate = sprintf("\x25\x73\45\x73", $redirectionVhostTemplate, $vhostTemplate); goto Ed8bf; af812: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto C155f; f0d1e: $siteEntity->setUserPassword($siteUserPassword); goto f65a3; c1431: $siteEntity->setPythonSettings($pythonSettingsEntity); goto c633a; A0976: $pythonSite->setUser($siteEntity->getUser()); goto a4fd3; a4fd3: $pythonSite->setUserPassword($siteEntity->getUserPassword()); goto Cf406; dda85: $pythonSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto b27a6; F9748: $subjectAlternativeNames = []; goto D510a; Cc8a2: $siteEntity = $this->siteEntityManager->createEntity(); goto fdc45; aeb73: $pythonSettingsEntity = new PythonSettingsEntity(); goto c1431; F95b4: if (true === isset($_ENV["\101\x50\120\137\x48\x54\x54\120\x33"]) && "\x74\162\x75\145" === $_ENV["\x41\x50\x50\x5f\x48\124\124\120\63"]) { goto E0233; } goto e01b0; Aac85: $certificateEntity->setDefaultCertificate(true); goto e3f36; Ae41d: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\56\57\56\x2e\x2f\x2e\x2e\x2f\162\x65\x73\157\165\x72\x63\x65\x73\x2f\156\x67\151\156\170\x2f\166\x68\x6f\x73\x74\137\x74\x65\155\x70\x6c\141\x74\x65\57\162\x65\x64\x69\x72\x65\x63\x74\55\150\x74\164\160\63"); goto B019d; Ae576: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto Aac85; fa5d7: $certificateEntity->setSite($siteEntity); goto E3041; fef77: $pythonSiteCreator->createLogrotateFile(); goto faba2; bca8f: $pythonSettingsConstraints = $this->validator->validate($pythonSettingsEntity); goto c17bf; a2f0c: $pythonSettingsEntity->setSite($siteEntity); goto E157c; e5dc0: E8e85: goto B788a; c3f95: $pythonSiteCreator->createUser(); goto f4e3a; E94ac: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto F9748; A7d38: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto d9fca; Be0ae: if (!(false === empty($redirectionVhostTemplate))) { goto b3eb6; } goto ecd57; fdc45: $siteEntity->setType(SiteEntity::TYPE_PYTHON); goto c1044; B788a: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto db825; d1c5d: $pythonSiteCreator->reloadNginxService(); goto be8fd; E157c: $siteConstraints = $this->validator->validate($siteEntity); goto bca8f; f77ea: $constraintViolationList->addAll($pythonSettingsConstraints); goto a54bb; be8fd: $pythonSiteCreator->resetPermissions(); goto B8568; d7fb4: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto Bb1df; e3f36: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto F623f; d9fca: $this->siteEntityManager->updateEntity($siteEntity); goto a4381; bed4f: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto d7fb4; faba2: $pythonSiteCreator->writePythonVersionFile(); goto C279a; dc44c: $pythonSettingsEntity->setPort($port); goto a2f0c; Cf406: $pythonSite->setDomainName($domainName); goto d28ad; Ed8bf: b3eb6: goto Deb8f; B30f1: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto D23af; F623f: $certificateEntity->setCsr($csr); goto F9bec; D5a34: goto F5f8a; goto b6722; Bed91: if (!(true === is_null($subdomain) || "\x77\167\x77" == $subdomain)) { goto E0f3a; } goto F95b4; b27a6: $pythonSiteCreator = new PythonSiteCreator($pythonSite); goto c3f95; E3041: $rsaKeyGenerator = new RsaKeyGenerator(); goto E94ac; A9fc9: $siteEntity->setVhostTemplate($vhostTemplate); goto aeb73; D23af: $vhostTemplate->resetProcessors(); goto bed4f; Ab6aa: throw new \Exception(implode("\x3b\x20", $errorMessages)); goto B9ba2; C279a: $pythonSiteCreator->createPrivateKeyAndCertificate(); goto F2e07; a54bb: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto Ab6aa; e123b: a7e05: goto a7120; cdb64: $pythonSite->setCertificate($certificateEntity); goto ed589; d28ad: $pythonSite->setRegistrableDomain($registrableDomain); goto bffb3; bffb3: $pythonSite->setSubdomain($subdomain); goto A2c22; ac546: $certificateEntity = $this->certificateEntityManager->createEntity(); goto fa5d7; A5997: $pythonSite = new PythonSite(); goto A0976; E45b6: $vhostTemplate->build(); goto A7d38; B9ba2: af64f: goto ac546; a7120: if (!(false === is_null($subdomain) && "\167\167\167" == $subdomain)) { goto E8e85; } goto c00ba; c1044: $siteEntity->setDomainName($domainName); goto e55df; A9ce6: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::PYTHON_VHOST_TEMPLATE_NAME); goto E9b8b; A2c22: $pythonSite->setRootDirectory($siteEntity->getRootDirectory()); goto cdb64; f4e3a: $pythonSiteCreator->createRootDirectory(); goto fef77; Ced6e: $certificateEntity->setCertificate($selfSignedCertificate); goto c7286; c7286: $siteEntity->setCertificate($certificateEntity); goto A5997; c00ba: $subjectAlternativeNames[] = $registrableDomain; goto e5dc0; Dae84: } private function createStaticSite(string $domainName, string $siteUser, string $siteUserPassword) : SiteEntity { goto af170; af170: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto c58ab; dff59: $vhostTemplate->build(); goto B587d; Bdb0d: $staticSite->setDomainName($domainName); goto b9f7b; ff071: goto A614d; goto ae87a; cbf08: A614d: goto D51c3; b8725: Ecfd7: goto Aaf8b; Eeba8: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto A0598; Aa169: $staticSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto A89e5; c58ab: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto a20d9; af4d5: if (true === isset($_ENV["\x41\x50\120\137\110\124\124\120\x33"]) && "\164\x72\165\145" === $_ENV["\x41\x50\120\x5f\x48\124\x54\x50\63"]) { goto f9019; } goto cdf8e; C41b6: $siteEntity->setDomainName($domainName); goto c85fe; a20d9: $subdomain = $resolvedDomainName->subDomain()->toString(); goto aef98; ea8b7: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto D795f; D6104: $vhostTemplate = sprintf("\x25\163\45\x73", $redirectionVhostTemplate, $vhostTemplate); goto bf9df; B9d8e: $siteEntity->setType(SiteEntity::TYPE_STATIC); goto C41b6; Aaf8b: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto Ec55b; b9b4d: return $siteEntity; goto e39c9; cd7e8: $vhostTemplate->resetProcessors(); goto ced93; f932d: Dd7c8: goto fece0; c26c4: $siteEntity = $this->siteEntityManager->createEntity(); goto B9d8e; B587d: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto c5efe; D31a0: $certificateEntity = $this->certificateEntityManager->createEntity(); goto F0ad9; F3f40: $siteEntity->setRootDirectory($domainName); goto C6c30; E2f01: $this->siteEntityManager->updateEntity($siteEntity); goto b9b4d; f648e: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\56\x2f\x2e\x2e\x2f\56\56\x2f\x72\145\163\157\x75\162\143\145\163\x2f\156\147\x69\156\x78\57\x76\150\x6f\163\164\137\164\x65\x6d\x70\x6c\141\x74\145\57\162\145\144\151\162\145\x63\164\x2d\150\x74\164\160\63"); goto cbf08; F0ad9: $certificateEntity->setSite($siteEntity); goto ea498; C11bd: $csr = $csrGenerator->generate(); goto Dacd6; c0e49: $siteEntity->setUserPassword($siteUserPassword); goto F3f40; F3d31: $siteEntity->setVhostTemplate($vhostTemplate); goto Bde50; b9f7b: $staticSite->setRegistrableDomain($registrableDomain); goto a683c; ae87a: f9019: goto f648e; cb9ca: $subjectAlternativeNames = []; goto A2239; c5efe: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto E2f01; Cb54d: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto Ff9e2; C08c5: $staticSiteCreator->createRootDirectory(); goto Dfee5; ac949: $staticSiteCreator->createPrivateKeyAndCertificate(); goto e712a; C0259: $certificateEntity->setCertificate($selfSignedCertificate); goto E902d; cbb0c: $staticSiteCreator->createUser(); goto C08c5; f09a4: $subjectAlternativeNames[] = sprintf("\167\x77\x77\x2e\x25\x73", $domainName); goto f932d; dd214: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto C0259; ced93: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto Cb54d; d7fb9: $constraintViolationList = new ConstraintViolationList(); goto e242d; A89e5: $staticSiteCreator = new StaticSiteCreator($staticSite); goto cbb0c; c4168: $staticSite->setUser($siteEntity->getUser()); goto Bbc86; C7bd9: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto c26c4; de7fc: $staticSite->setRootDirectory($siteEntity->getRootDirectory()); goto e1ea5; C6c30: if (!(true === is_null($subdomain) || "\x77\167\167" == $subdomain)) { goto Dc313; } goto af4d5; Ebf22: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::STATIC_SITE_VHOST_TEMPLATE_NAME); goto C7bd9; Dacd6: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto b8375; E902d: $siteEntity->setCertificate($certificateEntity); goto fba5d; D795f: $certificateEntity->setCsr($csr); goto dd214; Fdbb6: Dc313: goto F3d31; fba5d: $staticSite = new StaticSite(); goto c4168; c85fe: $siteEntity->setUser($siteUser); goto c0e49; bf9df: ab584: goto Fdbb6; b8375: $certificateEntity->setDefaultCertificate(true); goto ea8b7; f98db: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto cd7e8; c249b: if (!(false === empty($redirectionVhostTemplate))) { goto ab584; } goto D6104; Dfee5: $staticSiteCreator->createLogrotateFile(); goto Bf6fa; e1ea5: $staticSite->setCertificate($certificateEntity); goto Aa169; Da599: $subjectAlternativeNames[] = $registrableDomain; goto b8725; Fc1f2: $staticSiteCreator->resetPermissions(); goto F20ee; e242d: $constraintViolationList->addAll($siteConstraints); goto Eeba8; Bde50: $siteConstraints = $this->validator->validate($siteEntity); goto e662a; Ff117: Fcfe2: goto D31a0; fece0: if (!(false === is_null($subdomain) && "\167\x77\x77" == $subdomain)) { goto Ecfd7; } goto Da599; A0598: throw new \Exception(implode("\x3b\40", $errorMessages)); goto Ff117; e712a: $staticSiteCreator->createNginxVhost(); goto Dd642; Bf6fa: $staticSiteCreator->createIndexHtml(); goto ac949; cdf8e: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\56\56\57\56\56\57\x2e\56\x2f\x72\145\163\x6f\x75\x72\x63\x65\163\x2f\156\147\151\x6e\170\x2f\x76\150\x6f\163\x74\137\x74\x65\x6d\160\x6c\141\164\145\57\162\x65\144\x69\162\145\143\x74"); goto ff071; Ff9e2: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto dff59; Bbc86: $staticSite->setUserPassword($siteEntity->getUserPassword()); goto Bdb0d; F20ee: $vhostTemplate = new StaticVhostTemplate($staticSite); goto f98db; aef98: $subdomain = false === empty($subdomain) ? $subdomain : null; goto Ebf22; cf6d6: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto cb9ca; ea498: $rsaKeyGenerator = new RsaKeyGenerator(); goto cf6d6; D51c3: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto c249b; a683c: $staticSite->setSubdomain($subdomain); goto de7fc; Ec55b: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto C11bd; A2239: if (!(true === is_null($subdomain))) { goto Dd7c8; } goto f09a4; Dd642: $staticSiteCreator->reloadNginxService(); goto Fc1f2; e662a: if (!count($siteConstraints)) { goto Fcfe2; } goto d7fb9; e39c9: } private function createReverseProxySite(string $domainName, string $reverseProxyUrl, string $siteUser, string $siteUserPassword) : SiteEntity { goto A51cd; A51cd: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto c9d42; F5c6a: $reversProxySiteCreator->reloadNginxService(); goto Eca8a; bebb2: $subdomain = false === empty($subdomain) ? $subdomain : null; goto fb632; d00b5: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto D171b; bde7d: a127b: goto e5806; d9636: return $siteEntity; goto A6698; de478: $csr = $csrGenerator->generate(); goto F1903; Fec27: $reverseProxySite->setDomainName($domainName); goto Dac63; F1554: $reversProxySiteCreator->createRootDirectory(); goto be8c3; Eff4e: $siteEntity->setUser($siteUser); goto D1fec; b7bae: $subjectAlternativeNames = []; goto A11c5; E5c0c: C5970: goto a8187; Cda63: $reverseProxySite->setReverseProxyUrl($reverseProxyUrl); goto Fae1b; c9d42: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto baaff; A38ab: $siteEntity->setDomainName($domainName); goto Eff4e; Abd92: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto Ccef8; b42e5: $reversProxySiteCreator = new ReverseProxySiteCreator($reverseProxySite); goto fed01; b6fea: $reverseProxySite->setUserPassword($siteEntity->getUserPassword()); goto Fec27; E9a7c: $vhostTemplate->build(); goto d00b5; fab25: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto Ef85c; Efe81: goto A9c2d; goto d0067; be8c3: $reversProxySiteCreator->createLogrotateFile(); goto acecd; ac8c7: $subjectAlternativeNames[] = sprintf("\x77\167\x77\x2e\45\x73", $domainName); goto bde7d; F0181: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\56\x2f\56\56\57\56\x2e\x2f\x72\145\163\157\165\162\x63\145\163\x2f\x6e\147\x69\x6e\170\x2f\166\150\157\163\164\137\164\x65\x6d\x70\x6c\x61\x74\145\57\162\x65\144\151\x72\145\x63\x74"); goto Efe81; f7bef: $this->siteEntityManager->updateEntity($siteEntity); goto d9636; e3f6f: fda1a: goto bb0a1; c8115: $reverseProxySite = new ReverseProxySite(); goto D4ed0; bb147: $siteConstraints = $this->validator->validate($siteEntity); goto d4eca; C94d5: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto E9a7c; bb7b9: $reverseProxySite->setSubdomain($subdomain); goto Aa3cb; f9880: if (true === isset($_ENV["\x41\120\120\137\x48\x54\124\120\x33"]) && "\164\162\x75\x65" === $_ENV["\101\120\120\x5f\x48\x54\124\120\63"]) { goto ce555; } goto F0181; d0067: ce555: goto e7a6a; d83ee: $certificateEntity->setDefaultCertificate(true); goto e65ef; Ecc3a: $siteEntity->setRootDirectory($domainName); goto df1b9; Ef85c: throw new \Exception(implode("\x3b\40", $errorMessages)); goto E5c0c; D171b: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto f7bef; b805e: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto b7bae; e65ef: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto b67bc; A11c5: if (!(true === is_null($subdomain))) { goto a127b; } goto ac8c7; Cd5ce: $constraintViolationList = new ConstraintViolationList(); goto D0dd0; ff5dc: $reversProxySiteCreator->createNginxVhost(); goto F5c6a; f2614: $certificateEntity->setSite($siteEntity); goto c3fa6; a8187: $certificateEntity = $this->certificateEntityManager->createEntity(); goto f2614; D1fec: $siteEntity->setUserPassword($siteUserPassword); goto Ecc3a; Cb566: $certificateEntity->setCertificate($selfSignedCertificate); goto a850b; a49f5: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto Cb566; bafd5: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto D6f45; F1903: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto d83ee; F6724: $reverseProxySite->setCertificate($certificateEntity); goto Cda63; A852b: if (!(false === empty($redirectionVhostTemplate))) { goto fda1a; } goto Edc83; Fe795: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto C2e23; Ccef8: $siteEntity = $this->siteEntityManager->createEntity(); goto d47a5; Ad5eb: $siteEntity->setVhostTemplate($vhostTemplate); goto bb147; fef0f: $subjectAlternativeNames[] = $registrableDomain; goto C9c79; Aa3cb: $reverseProxySite->setRootDirectory($siteEntity->getRootDirectory()); goto F6724; C9c79: E25b4: goto bafd5; ce1ef: $vhostTemplate->resetProcessors(); goto Fe795; fb632: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::REVERSE_PROXY_VHOST_TEMPLATE_NAME); goto Abd92; D6f45: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto de478; Ad0e4: if (!(true === is_null($subdomain) || "\x77\167\167" == $subdomain)) { goto dea52; } goto f9880; fed01: $reversProxySiteCreator->createUser(); goto F1554; Dac63: $reverseProxySite->setRegistrableDomain($registrableDomain); goto bb7b9; a850b: $siteEntity->setCertificate($certificateEntity); goto c8115; bb0a1: dea52: goto Ad5eb; e7a6a: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\56\56\57\x2e\x2e\x2f\x2e\56\57\162\x65\163\x6f\165\x72\143\145\163\x2f\156\x67\151\x6e\x78\x2f\x76\x68\x6f\x73\164\x5f\164\145\155\x70\x6c\x61\164\145\x2f\162\145\x64\x69\162\x65\x63\x74\55\150\164\164\x70\63"); goto Db230; e5806: if (!(false === is_null($subdomain) && "\167\167\167" == $subdomain)) { goto E25b4; } goto fef0f; d4eca: if (!count($siteConstraints)) { goto C5970; } goto Cd5ce; Db230: A9c2d: goto F69be; b6500: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto ce1ef; b67bc: $certificateEntity->setCsr($csr); goto a49f5; Fae1b: $reverseProxySite->setVhostTemplate($siteEntity->getVhostTemplate()); goto b42e5; acecd: $reversProxySiteCreator->createPrivateKeyAndCertificate(); goto ff5dc; D4ed0: $reverseProxySite->setUser($siteEntity->getUser()); goto b6fea; Eca8a: $reversProxySiteCreator->resetPermissions(); goto Baaa8; baaff: $subdomain = $resolvedDomainName->subDomain()->toString(); goto bebb2; C2e23: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto C94d5; Baaa8: $vhostTemplate = new ReverseProxyVhostTemplate($reverseProxySite); goto b6500; D0dd0: $constraintViolationList->addAll($siteConstraints); goto fab25; Edc83: $vhostTemplate = sprintf("\x25\163\45\163", $redirectionVhostTemplate, $vhostTemplate); goto e3f6f; df1b9: $siteEntity->setReverseProxyUrl($reverseProxyUrl); goto Ad0e4; c3fa6: $rsaKeyGenerator = new RsaKeyGenerator(); goto b805e; d47a5: $siteEntity->setType(SiteEntity::TYPE_REVERSE_PROXY); goto A38ab; F69be: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto A852b; A6698: } public function installedWordPressSite(Request $request) : Response { goto A54e3; d3c9d: if (false === empty($siteCredentials)) { goto F0329; } goto fe7d5; f3e8b: return $response; goto C9ccc; C9ccc: c5391: goto a2062; a2062: return $response; goto D6b1d; fcb13: $siteCredentials = $this->renderSiteCredentials($siteCredentials); goto Ec7b8; Ec7b8: $response = $this->render("\106\x72\x6f\x6e\164\145\x6e\x64\57\123\x69\x74\145\57\116\145\x77\57\x77\x6f\x72\x64\x70\162\x65\163\x73\x2d\151\x6e\163\x74\141\x6c\154\145\x64\x2e\x68\164\155\154\56\x74\x77\x69\147", ["\x73\x69\164\x65\x43\162\145\144\x65\156\x74\x69\141\154\x73" => $siteCredentials]); goto f3e8b; fe7d5: $response = $this->redirect($this->generateUrl("\x63\154\160\137\163\151\164\x65\x73")); goto Dfb6b; Bb0af: $siteCredentials = $session->get("\x73\151\164\145\x43\x72\x65\x64\145\156\164\151\141\x6c\x73"); goto d3c9d; A54e3: $session = $request->getSession(); goto Bb0af; Dfb6b: goto c5391; goto E1f52; E1f52: F0329: goto fcb13; D6b1d: } private function renderSiteCredentials(array $siteCredentials) : string { goto fbc86; Dc789: return $renderedCredentials; goto B2dd0; A4ebd: d10e5: goto Dc789; fbc86: $renderedCredentials = ''; goto bdd22; bdd22: foreach ($siteCredentials as $section => $sectionData) { goto bab44; bab44: $renderedCredentials .= $section . PHP_EOL; goto A6b41; B7590: $renderedCredentials .= PHP_EOL; goto Daa92; C788b: foreach ($sectionData as $key => $value) { $renderedCredentials .= sprintf("\x25\163\x3a\x20\x25\163", $key, $value) . PHP_EOL; e8c1d: } goto B3ac0; B3ac0: fa517: goto B7590; Daa92: f908a: goto E6135; A6b41: $renderedCredentials .= "\55\x2d\55\x2d\x2d\x2d\55\x2d\55\55\55\55\x2d\x2d\x2d\x2d\55\x2d\x2d\55\x2d\55\55\x2d\x2d\55\x2d\x2d\x2d\x2d\55\x2d\55\x2d\x2d\55\55\x2d\55\55\x2d\55\55\55\x2d\55\x2d\55\55\55\x2d\55\55\x2d" . PHP_EOL; goto C788b; E6135: } goto A4ebd; B2dd0: } private function getVhostTemplateEntity(string $name) : mixed { goto f1882; F69d3: return $vhostTemplateEntity; goto b7786; dbcf5: D32c6: goto F69d3; f1882: $vhostTemplateEntity = $this->vhostTemplateEntityManager->findOneByName($name); goto F4aee; F4aee: if (!(true === is_null($vhostTemplateEntity))) { goto D32c6; } goto d9dcd; d9dcd: throw new \Exception($this->translator->trans("\126\x68\x6f\163\164\40\124\x65\x6d\x70\154\x61\x74\145\x20\x25\166\x68\x6f\x73\164\124\145\x6d\160\154\141\164\145\45\40\x64\157\145\x73\x20\156\157\x74\40\145\170\x69\163\164\x2e", ["\45\166\150\x6f\163\164\124\145\155\x70\154\141\x74\145\45" => $name])); goto dbcf5; b7786: } private function getConstraintViolationListErrorMessages(ConstraintViolationList $constraintViolationList) : array { goto e09ea; e09ea: $errorMessages = []; goto C69a1; C69a1: foreach ($constraintViolationList as $constraint) { $errorMessages[] = sprintf("\45\163\72\40\x25\163", $constraint->getPropertyPath(), $constraint->getMessage()); dc95b: } goto cbfae; f6677: return $errorMessages; goto fb1b0; cbfae: A813b: goto f6677; fb1b0: } private function generateNameFromDomainName(string $domainName) : string { goto E37e2; d6f1d: goto Eb708; goto e3101; A5e3a: $name = sprintf("\45\163\55\45\x73", $explodedRegistrableDomain[0] ?? '', implode("\x2d", $explodedSubdomain)); goto c7ab9; e3101: E75a7: goto b200b; E37e2: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto efe35; efe35: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto F3617; F6ba8: Eb708: goto Fe7f1; f25d5: $name = $explodedRegistrableDomain[0] ?? ''; goto B63ec; Fe7f1: return $name; goto D0491; B63ec: if (!(false === empty($subdomain))) { goto E9942; } goto C4893; C4893: $explodedSubdomain = explode("\56", $subdomain); goto A5e3a; f1890: if ("\167\x77\x77" == $subdomain) { goto E75a7; } goto f25d5; B98ea: $explodedRegistrableDomain = explode("\x2e", $registrableDomain); goto f1890; c7ab9: E9942: goto d6f1d; F3617: $subdomain = $resolvedDomainName->subDomain()->toString(); goto B98ea; b200b: $name = $explodedRegistrableDomain[0] ?? ''; goto F6ba8; D0491: } private function getUniqueDatabaseName(string $databaseName) : string { goto C6521; A2f23: return $uniqueDatabaseName; goto dcc87; d83f5: $databases = $databaseConnection->getDatabases(); goto Fae7e; fde24: fb1e1: goto A2f23; c10e6: $databaseExists = in_array($uniqueDatabaseName, $databases); goto F4b93; a1f1b: $i = 0; goto d43ba; Fae7e: d1d08: goto c3fb2; C6521: $uniqueDatabaseName = $databaseName; goto D8e10; D8e10: if (!(strlen($uniqueDatabaseName) <= 3)) { goto bfa94; } goto F2875; F2875: $uniqueDatabaseName = sprintf("\45\x73\55\x64\142", $uniqueDatabaseName); goto f3348; c3e3c: if (true === $databaseExists) { goto d1d08; } goto fde24; e1f23: F6d4d: goto c10e6; f3348: bfa94: goto a1f1b; a422b: $uniqueDatabaseName = sprintf("\x25\x73\x2d\45\x73", $databaseName, $i); goto e1f23; Afdb1: $databaseConnection = new DatabaseConnection($activeDatabaseServerEntity); goto d83f5; F4b93: $i++; goto c3e3c; c3fb2: if (!(0 < $i)) { goto F6d4d; } goto a422b; d43ba: $activeDatabaseServerEntity = $this->databaseServerEntityManager->getActiveDatabaseServer(); goto Afdb1; dcc87: } private function getUniqueDatabaseUserName(string $databaseUserName) : string { goto E9095; D7117: $i = 0; goto B2b9b; C7be9: $i++; goto e3721; F03b0: $uniqueDatabaseUserName = sprintf("\45\163\55\144\142", $uniqueDatabaseUserName); goto ff654; b70cc: return $uniqueDatabaseUserName; goto fc9e9; E9095: $uniqueDatabaseUserName = $databaseUserName; goto b4aee; b4aee: if (!(strlen($uniqueDatabaseUserName) <= 3)) { goto Ac80d; } goto F03b0; F0ae8: $uniqueDatabaseUserName = mb_substr($uniqueDatabaseUserName, 0, 32); goto b70cc; b8209: $databaseUserEntity = $this->databaseUserEntityManager->findOneByUserName($uniqueDatabaseUserName); goto C7be9; ac441: if (!(0 < $i)) { goto Aae7c; } goto b9184; ff654: Ac80d: goto D7117; b9a63: Aea05: goto F0ae8; b9184: $uniqueDatabaseUserName = sprintf("\45\163\x2d\x25\x73", $databaseUserName, $i); goto bf018; e3721: if (false === is_null($databaseUserEntity)) { goto a8c2d; } goto b9a63; bf018: Aae7c: goto b8209; B2b9b: a8c2d: goto ac441; fc9e9: } private function hasPermissions() : bool { goto e9d75; Ad25c: if (!(UserEntity::ROLE_USER != $user->getRole())) { goto a0206; } goto Ba5cf; e9d75: $user = $this->getUser(); goto Ad25c; Ba5cf: return true; goto B08d3; B08d3: a0206: goto Dfcc8; Dfcc8: return false; goto cd130; cd130: } }
